﻿using Nest;
using SMP.Data.TimeTask;
using SMP.Model.Dto.DataInterconnection;
using SMP.Model.Enums;
using SMP.Model.Finance;
using SMP.Model.TimeTask;
using SMP.Service.Finance;
using SMP.Service.ThirdPartyRequest;
using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using XYY.Common.Standard;

namespace SMP.Service.TimeTask
{
    #region Iservices
    public interface IFinanceTaskService
    {
        Task<PageResult<Finance_TimeTask>> QueryPage(QueryModel query);
        Task<long> Create(Finance_TimeTask model);
        Task<IEnumerable<Finance_TimeTask>> QueryWaitRead();
        Task<bool> ExecuteTaskJob(int taskId, DateTime startTime, DateTime endTime);
        Task RetryExecute(int id, TaskTypeEnum taskTypeEnum);
    }
    #endregion
    #region services
    public class FinanceTaskService : IFinanceTaskService
    {

        private readonly IFinance_TimeTaskRepository _repository;
        private readonly IWMSDataInterconnectionService _wMSDataInterconnection;
        private readonly IOrderCostService _orderCostService;
        public FinanceTaskService(IFinance_TimeTaskRepository repository, IWMSDataInterconnectionService wMSDataInterconnection, IOrderCostService orderCostService)
        {
            _repository = repository;
            this._wMSDataInterconnection = wMSDataInterconnection;
            this._orderCostService = orderCostService;
        }
        public async Task<PageResult<Finance_TimeTask>> QueryPage(QueryModel query)
        {
            var result = await _repository.QueryPager2Async(query);
            return result;
        }
        public async Task RetryExecute(int id, TaskTypeEnum taskTypeEnum)
        {
            string sql = $" update Finance_TimeTask set TaskStatus={ (int)FinanceEnum.WaitRead},TaskType={ (int)taskTypeEnum} where Id={id}";
            await _repository.ExecuteSqlAsync(sql);
        }
        public async Task<long> Create(Finance_TimeTask model)
        {
            return await _repository.InsertAsync(model);
        }
        public async Task<IEnumerable<Finance_TimeTask>> QueryWaitRead()
        {
            var result = await _repository.QueryAsync(x => x.TaskStatus == (int)FinanceEnum.WaitRead);
            //一次只取50条任务
            var tempResult = result.Take(50);
            List<int> Ids = tempResult.Select(x => x.Id).Distinct().ToList();
            await _repository.UpdateTaskStatus(FinanceEnum.WaitExecute, Ids);
            return result;
        }
        public async Task<bool> ExecuteTaskJob(int taskId, DateTime startTime, DateTime endTime)
        {
            Stopwatch watch = new Stopwatch();
            watch.Start();
            var dbModel = await _repository.GetAsync(taskId);
            List<Logistisc_Order> orders = new List<Logistisc_Order>();
            dbModel.TaskStatus = FinanceEnum.Executing;
            try
            {
                switch (dbModel.TaskType)
                {
                    case TaskTypeEnum.NormalTime:
                        orders = await ExecuteByNormalTime(dbModel, startTime, endTime);
                        break;
                    case TaskTypeEnum.DetailScope:
                        orders = await ExecuteByDetailScope(dbModel);
                        break;
                }
                if (orders != null)
                {
                    //成本预估操作
                    bool result= await _orderCostService.EestimateOrderEndCost(dbModel,orders);
                    if (result) { dbModel.ResultMessage = "执行成功！"; dbModel.IsSucceed = true; dbModel.TaskStatus = FinanceEnum.Succeed; }
                    else { dbModel.ResultMessage = "执行过程中有计算异常数据！"; dbModel.IsSucceed = false; dbModel.TaskStatus = FinanceEnum.Fail; }
                }
                else
                {
                    dbModel.ResultMessage = "待执行订单获取失败！"; dbModel.IsSucceed = false; dbModel.TaskStatus = FinanceEnum.Fail;
                }
            }
            catch (Exception ex)
            {
                dbModel.ResultMessage = "执行异常！请整个任务重试！" + ex.Message; dbModel.IsSucceed = false; dbModel.TaskStatus = FinanceEnum.Fail;
            }
            finally
            {
                watch.Stop();
                dbModel.RetryTotal++;
                dbModel.TimeConsuming = watch.ElapsedMilliseconds;
                await _repository.UpdateAsync(dbModel);
            }
            return dbModel.IsSucceed ?? false;
        }
        #region 执行方式
        private async Task<List<Logistisc_Order>> ExecuteByNormalTime(Finance_TimeTask dbModel, DateTime startTime, DateTime endTime)
        {
            ThirdPartyData_Input input = new ThirdPartyData_Input() { startTime = startTime, endTime = endTime };
            var thirdData = await _wMSDataInterconnection.PullOrderInfo(input);
            if (thirdData == null || thirdData.orders.Count() == 0) { dbModel.ResultMessage = "时间范围内无数据！"; dbModel.IsSucceed = true; dbModel.TaskStatus = FinanceEnum.Succeed; return null; }
            else { dbModel.OrderCount = thirdData.orders.Count(); }
            await _orderCostService.SyncOrderInfo(thirdData.orders);
            return thirdData.orders.ToList();
        }
        private async Task<List<Logistisc_Order>> ExecuteByDetailScope(Finance_TimeTask dbModel)
        {
            var detail = await _repository.GetTaskDetails(dbModel.Id);
            if (detail == null || detail.Count() == 0)
            {
                dbModel.ResultMessage = $"任务记录异常！";
                if (!string.IsNullOrEmpty(dbModel.LadingBillNumber))
                {
                    dbModel.ResultMessage += $"请重新保存提单[{dbModel.LadingBillNumber}]";
                }
                dbModel.IsSucceed = false; dbModel.TaskStatus = FinanceEnum.Fail; return null;
            }
            var systemNos = detail.Select(x => x.SystemNo).Distinct().ToList();
            var orders = await _orderCostService.GetOrdersBySystemNos(systemNos);
            return orders.ToList();
        }
        #endregion
    }
    #endregion

}
